@@ -1060,11 +1060,11 @@ to a temporary variable.
 =item base_comp
 
 Returns the current base component for method and attributes.
-Generally set to the original page component; however, if you invoke
-call_method on a component, C<base_comp> is dynamically set to that
-component until call_method exits. See L<Object-Oriented Techniques in
-the I<Component Developer's Guide>|HTML::Mason::Devel/"Object-Oriented
-Techniques"> for examples of usage.
+Initially, the base component is the same as the requested component
+(returned by C<< $m->request_comp >>.  However, whenever a component
+call is made, the base component changes to the called component,
+unless the component call was made uses a component object for its
+first argument, or the call starts with SELF: or PARENT:.
 
 =for html <a name="item_cache">
 
@@ -1418,12 +1418,15 @@ Returns the Interp object associated with this request.
 
 =for html <a name="item_make_subrequest">
 
-=item make_subrequest (parameters)
+=item make_subrequest (comp => path, args => arrayref, other parameters)
 
 This method creates a new Request object which inherits its parent's
 settable properties, such as C<autoflush> and C<out_method>.  These
 values may be overridden by passing parameters to this method.
 
+The "comp" parameter is required, while all other parameters are
+optional.
+
 See the L<Subrequests section in the I<Component Developer's
 Guide>|HTML::Mason::Devel/"Subrequests"> for more details about the
 subrequest feature.
